@@ -82,6 +82,7 @@ enum {
 };
 
 static void egg_toolbar_init       (EggToolbar      *toolbar);
+static void egg_toolbar_finalize   (GObject         *object);
 static void egg_toolbar_class_init (EggToolbarClass *klass);
 
 static void egg_toolbar_set_property (GObject      *object,
@@ -283,6 +284,7 @@ egg_toolbar_class_init (EggToolbarClass *klass)
   
   gobject_class->set_property = egg_toolbar_set_property;
   gobject_class->get_property = egg_toolbar_get_property;
+  gobject_class->finalize = egg_toolbar_finalize;
 
   widget_class->button_press_event = egg_toolbar_button_press;
   widget_class->expose_event = egg_toolbar_expose;
@@ -488,6 +490,21 @@ egg_toolbar_class_init (EggToolbarClass *klass)
   add_ctrl_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_LEFT);
 }
 
+static void
+egg_toolbar_finalize (GObject *object)
+{
+  EggToolbar *toolbar = EGG_TOOLBAR (object);
+  EggToolbarPrivate *priv;
+
+  if (toolbar->tooltips)
+    g_object_unref (GTK_OBJECT (toolbar->tooltips));
+
+  priv = g_object_get_data (object, PRIVATE_KEY);
+  g_free (priv);
+  
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 egg_toolbar_init (EggToolbar *toolbar)
 {
@@ -562,6 +579,25 @@ toolbar_item_visible (EggToolbar  *toolbar,
   return FALSE;
 }
 
+static gboolean
+toolbar_item_is_homogeneous (EggToolbar  *toolbar,
+                             EggToolItem *item)
+{
+  gboolean result = FALSE;
+                                                                                                                             
+  if (item->homogeneous && !EGG_IS_SEPARATOR_TOOL_ITEM (item))
+    result = TRUE;
+                                                                                                                             
+  if (item->is_important &&
+      toolbar->style == GTK_TOOLBAR_BOTH_HORIZ &&
+      toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      result = FALSE;
+    }
+                                                                                                                             
+  return result;
+}
+
 static void
 egg_toolbar_set_property (GObject     *object,
 			  guint        prop_id,
@@ -822,7 +858,7 @@ egg_toolbar_size_request (GtkWidget      *widget,
       max_child_width = MAX (max_child_width, requisition.width);
       max_child_height = MAX (max_child_height, requisition.height);
 
-      if (EGG_TOOL_ITEM (item)->homogeneous && GTK_BIN (item)->child)
+      if (toolbar_item_is_homogeneous (toolbar, item) && GTK_BIN (item)->child)
 	{
 	  max_homogeneous_child_width = MAX (max_homogeneous_child_width, requisition.width);
 	  max_homogeneous_child_height = MAX (max_homogeneous_child_height, requisition.height);
@@ -948,14 +984,14 @@ get_item_size (EggToolbar *toolbar,
   
   if (toolbar->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      if (item->homogeneous)
+      if (toolbar_item_is_homogeneous (toolbar, item))
 	return toolbar->button_maxw;
       else
 	return requisition.width;
     }
   else
     {
-      if (item->homogeneous)
+      if (toolbar_item_is_homogeneous (toolbar, item))
 	return toolbar->button_maxh;
       else
 	return requisition.height;
